from qrunner.utils.faker.factory import Factory
from qrunner.utils.faker.generator import Generator
from qrunner.utils.faker.proxy import Faker

VERSION = "13.3.5"

__all__ = ("Factory", "Generator", "Faker")



from datetime import date, datetime, timedelta
from typing import Sequence, TypeVar, Union

try:
    from typing import Literal  # type: ignore
except ImportError:
    from typing_extensions import Literal  # type: ignore

DateParseType = Union[date, datetime, timedelta, str, int]
HueType = TypeVar("HueType", str, float, Sequence[int])
GenderType = TypeVar("GenderType", bound=Literal["M", "F"])



import os
import json
import yaml


local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.yml')
        with open(self.file_path, "r", encoding="utf-8") as f:
            self.yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)

    def get_all(self):
        return self.yaml_data

    def get_item(self, module, key):
        return self.yaml_data[module][key]

    def set_item(self, module, key, value):
        self.yaml_data[module][key] = value
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(self.yaml_data, f)


conf = Config()


if __name__ == '__main__':
    conf.get_all()
    conf.set_item('common', 'platform', 'android')
    conf.set_item('common', 'platform', 'ios')
    print(conf.get_item('app', 'android_alert'))
    print(conf.get_item('common', 'platform'))






from qrunner.running.runner import main
from qrunner.case import TestCase, Page
from qrunner.core.api.request import HttpRequest, ResponseResult
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.h5.driver import H5Driver
from qrunner.core.web.driver import WebDriver
from qrunner.core.web.element import WebElement
from qrunner.utils.log import logger
from qrunner.utils.decorate import *
from qrunner.utils.mock import mock, mock_en, RandomData, CommonData
from qrunner.utils.config import conf

__version__ = "0.9.36"
__description__ = "全栈自动化测试框架"




import operator

from collections import OrderedDict

from .. import Provider as BaseProvider


def nit_check_digit(nit: str) -> str:
    """
    Calculate the check digit of a NIT.

    The check digit is calculated by multiplying the reversed digits of a NIT
    by (3, 7, 13, 17, 19, 23, 29, 37, 41, 43, 47, 53, 59, 67, 71), respectively,
    adding the results and applying MOD 11. If the result is greater than or equal
    to 2, the check digit is 11 minus the result. Otherwise, the check digit is the
    result.
    """
    reversed_nit = nit[::-1]
    digits = (int(digit) for digit in reversed_nit)
    multipliers = (3, 7, 13, 17, 19, 23, 29, 37, 41, 43, 47, 53, 59, 67, 71)
    value = sum(map(operator.mul, digits, multipliers)) % 11
    if value >= 2:
        value = 11 - value
    return str(value)


class Provider(BaseProvider):

    nuip_formats = OrderedDict(
        [
            ("10########", 0.25),
            ("11########", 0.25),
            ("12########", 0.1),
            ("%!######", 0.4),
        ]
    )

    legal_person_nit_formats = [
        "8########",
        "9########",
    ]

    def nuip(self) -> str:
        """
        https://es.wikipedia.org/wiki/C%C3%A9dula_de_Ciudadan%C3%ADa_(Colombia)
        :example: '1095312769'
        """
        return self.numerify(self.random_element(self.nuip_formats))

    natural_person_nit = nuip

    def natural_person_nit_with_check_digit(self) -> str:
        """
        :example: '1095312769-0'
        """
        nit = self.natural_person_nit()
        check_digit = nit_check_digit(nit)
        return f"{nit}-{check_digit}"

    def legal_person_nit(self) -> str:
        """
        https://es.wikipedia.org/wiki/N%C3%BAmero_de_Identificaci%C3%B3n_Tributaria
        :example: '967807269'
        """
        return self.numerify(self.random_element(self.legal_person_nit_formats))

    def legal_person_nit_with_check_digit(self) -> str:
        """
        :example: '967807269-7'
        """
        nit = self.legal_person_nit()
        check_digit = nit_check_digit(nit)
        return f"{nit}-{check_digit}"



# coding=utf-8
import json
import os
import sys
import pytest
from qrunner.utils.log import logger
from qrunner.utils.config import conf


class TestMain(object):
    """
    Support for app and web
    """
    def __init__(self,
                 platform='api',
                 serial_no=None,
                 pkg_name=None,
                 browser_name='chrome',
                 case_path='test_dir',
                 rerun=0,
                 concurrent=False,
                 base_url=None,
                 headers=None,
                 login_key=None,
                 timeout=30,
                 report_path='report'
                 ):
        """
        :param platform str: 平台，如web、android、ios、api
        :param serial_no str: 设备id，如UJK0220521066836、00008020-00086434116A002E
        :param pkg_name str: 应用包名，如com.qizhidao.clientapp、com.qizhidao.company
        :param browser_name str: 浏览器类型，如chrome、其他暂不支持
        :param case_path str: 用例路径
        :param rerun int: 失败重试次数
        :param concurrent bool: 是否需要并发执行，只支持platform为browser的情况
        :@param base_url str: 接口host
        "@param headers dict: 额外的请求头，{
            "accessToken": "xxxx",
            "signature": "xxxx"
        }
        :@param login_key list：headers中跟登录相关的key
        :@param timeout int: 接口请求超时时间
        :@param report_path str: 报告生成目录
        """

        self.platform = platform
        self.serial_no = serial_no
        self.pkg_name = pkg_name
        self.browser_name = browser_name
        self.case_path = case_path
        self.rerun = str(rerun)
        self.concurrent = concurrent
        self.base_url = base_url
        self.headers = headers
        self.login_key = login_key
        self.timeout = str(timeout)
        self.report_path = report_path

        # 将数据写入全局变量
        if self.platform is not None:
            conf.set_item('common', 'platform', self.platform)
        if self.serial_no is not None:
            conf.set_item('app', 'serial_no', self.serial_no)
        if self.pkg_name is not None:
            conf.set_item('app', 'pkg_name', self.pkg_name)
        if self.browser_name is not None:
            conf.set_item('web', 'browser_name', self.browser_name)
        if self.base_url is not None:
            conf.set_item('common', 'base_url', self.base_url)
        else:
            if self.platform == 'api' or self.platform == 'web':
                print('base_url未配置，请配置后重新执行~')
                sys.exit()
        if self.headers is not None:
            conf.set_item('common', 'headers', headers)
        if login_key is not None:
            conf.set_item('api', 'login_key', login_key)
        if self.timeout is not None:
            conf.set_item('common', 'timeout', self.timeout)
        conf.set_item('common', 'report_path', self.report_path)

        # 执行用例
        logger.info('执行用例')
        logger.info(f'平台: {self.platform}')
        cmd_list = [
            '-sv',
            '--reruns', self.rerun,
            '--alluredir', 'allure-results', '--clean-alluredir'
        ]
        if self.case_path:
            cmd_list.insert(0, self.case_path)
        if self.concurrent:
            if self.platform == 'web' or self.platform == 'api':
                cmd_list.insert(1, '-n')
                cmd_list.insert(2, 'auto')
                cmd_list.insert(3, '--dist=loadscope')
            else:
                logger.info(f'{self.platform}平台不支持并发执行')
                sys.exit()
        logger.info(cmd_list)
        pytest.main(cmd_list)

        # 用例完成后操作
        conf.set_item('common', 'headers', {})  # 清除登录态
        os.system(f'allure generate allure-results -o {self.report_path} --clean')  # 生成报告


main = TestMain


if __name__ == '__main__':
    main()




import inspect
import time
import platform
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.select import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.remote.webelement import WebElement as WE
from qrunner.utils.log import logger
from qrunner.utils.exceptions import ElementTypeError, NoSuchElementException
from qrunner.core.web.driver import WebDriver


# 支持的定位方式
LOC_LIST = {
    'id_': By.ID,
    'name': By.NAME,
    'link_text': By.LINK_TEXT,
    'tag_name': By.TAG_NAME,
    'partial_link_text': By.PARTIAL_LINK_TEXT,
    'class_name': By.CLASS_NAME,
    'xpath': By.XPATH,
    'css': By.CSS_SELECTOR
}


class WebElement:
    """
    Web元素定义
    """
    def __init__(self, driver, **kwargs):
        self._index = kwargs.pop('index', 0)
        self.desc = kwargs.pop('desc', '')

        if not kwargs:
            raise ElementTypeError('请输入定位方式')

        if len(kwargs) > 1:
            raise ElementTypeError('请仅指定一种定位方式')

        self.k, self.v = next(iter(kwargs.items()))
        # print(self.k, self.v)
        if self.k not in LOC_LIST.keys():
            raise ElementTypeError(f'不支持的定位方式: {self.k}，仅支持: {LOC_LIST.keys()}')

        self._kwargs = kwargs
        self._element = None
        self.driver: WebDriver = driver
        self.d = self.driver.d

    def _wait(self, timeout=3):
        try:
            WebDriverWait(self.d, timeout=timeout)\
                .until(EC.visibility_of_element_located((LOC_LIST[self.k], self.v)))
            return True
        except Exception:
            return False

    def _find_element(self, retry=3, timeout=5):
        while not self._wait(timeout=timeout):
            if retry > 0:
                retry -= 1
                logger.info(f'重试 查找元素 {self._kwargs}')
                time.sleep(2)
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】Not found element {self._kwargs}')
                return None
        elements = self.d.find_elements(LOC_LIST[self.k], self.v)
        return elements

    def show_element(self, elem):
        """
                Show the elements of the operation
                :param elem:
                """
        style_red = 'arguments[0].style.border="2px solid #FF0000"'
        style_blue = 'arguments[0].style.border="2px solid #00FF00"'
        style_null = 'arguments[0].style.border=""'
        for _ in range(2):
            self.driver.execute_js(style_red, elem)
            time.sleep(0.1)
            self.driver.execute_js(style_blue, elem)
            time.sleep(0.1)
        self.driver.execute_js(style_blue, elem)
        time.sleep(0.3)
        self.driver.execute_js(style_null, elem)

    def get_elements(self, retry=3, timeout=3):
        elements = self._find_element(retry=retry, timeout=timeout)
        if elements is None:
            self.driver.screenshot(f'[控件 {self.desc} 定位失败]')
            raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
        else:
            element: WE = elements[self._index]
            self.show_element(element)
            # self.driver.screenshot(f'{loc}-定位成功')
        return elements

    def get_element(self, retry=3, timeout=3):
        elements = self._find_element(retry=retry, timeout=timeout)
        if elements is None:
            self.driver.screenshot(f'[控件 {self.desc} 定位失败]')
            raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
        else:
            element: WE = elements[self._index]
        # else:
        #     element: WE = elements[self._index]
        #     self.show_element(element)
            # self.driver.screenshot(f'[{self.desc}]-[定位成功]')
        return element

    def exists(self, timeout=1):
        logger.info(f'判断元素: {self._kwargs} 是否存在')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            # self.driver.screenshot(f'元素定位失败')
            return False
        return True

    def click(self):
        logger.info(f'点击元素: {self._kwargs}')
        self.get_element().click()

    def click_exists(self):
        logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
        if self.exists():
            self.click()

    def slow_click(self):
        logger.info(f'移动到元素{self._kwargs}，然后点击')
        elem = self.get_element()
        ActionChains(self.d).move_to_element(elem).click(elem).perform()

    def right_click(self):
        logger.info(f'右键元素{self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).context_click(elem).perform()

    def move_to_elem(self):
        logger.info(f'鼠标移动到元素{self._kwargs}上')
        elem = self.get_element()
        ActionChains(self.d).move_to_element(elem).perform()

    def click_and_hold(self):
        logger.info(f'长按元素: {self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).click_and_hold(elem).perform()

    def drag_and_drop(self, x, y):
        logger.info(f'拖动元素{self._kwargs}到坐标{x},{y}')
        elem = self.get_element()
        action = ActionChains(self.d)
        action.drag_and_drop_by_offset(elem, x, y).perform()

    def double_click(self):
        logger.info(f'双击元素: {self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).double_click(elem).perform()

    def set_text(self, text):
        logger.info(f'点击元素: {self._kwargs}，然后输入: {text}')
        self.get_element().send_keys(text)

    def set_and_enter(self, text):
        logger.info(f'往 {self._kwargs} 输入 {text} 并回车')
        element = self.get_element()
        element.send_keys(text)
        element.send_keys(Keys.ENTER)

    def clear_text(self):
        logger.info(f'清空文本')
        self.get_element().clear()

    def enter(self):
        logger.info(f'选中元素{self._kwargs}点击enter')
        self.get_element().send_keys(Keys.ENTER)

    def select_all(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+a.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "a")
        else:
            self.get_element().send_keys(Keys.CONTROL, "a")

    def cut(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+x.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "x")
        else:
            self.get_element().send_keys(Keys.CONTROL, "x")

    def copy(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+c.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "c")
        else:
            self.get_element().send_keys(Keys.CONTROL, "c")

    def paste(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+v.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "v")
        else:
            self.get_element().send_keys(Keys.CONTROL, "v")

    def backspace(self) -> None:
        logger.info(f"选中元素{self._kwargs}, backspace.")
        self.get_element().send_keys(Keys.BACKSPACE)

    def delete(self) -> None:
        logger.info(f"选中元素{self._kwargs}, delete.")
        self.get_element().send_keys(Keys.DELETE)

    def tab(self) -> None:
        logger.info(f"选中元素{self._kwargs}, tab.")
        self.get_element().send_keys(Keys.TAB)

    def space(self) -> None:
        logger.info(f"选中元素{self._kwargs}, space.")
        self.get_element().send_keys(Keys.SPACE)

    @property
    def rect(self):
        """获取的坐标位置不对，截图会偏"""
        # logger.info(f"获取元素 {self._kwargs}的坐标")
        # bounds = self.get_element().rect
        # logger.debug(f'rect: {bounds}')
        # x = bounds['x']
        # y = bounds['y']
        # width = bounds['width']
        # height = bounds['height']
        # return [x, y, width, height]
        return None

    def get_attr(self, attr_name):
        logger.info(f'获取属性{attr_name}的值')
        value = self.get_element().get_attribute(attr_name)
        logger.info(value)
        return value

    def get_display(self):
        logger.info(f'获取元素{self._kwargs}的display属性')
        displayed = self.get_element().is_displayed()
        logger.info(displayed)
        return displayed

    @property
    def text(self):
        logger.info(f'获取元素 {self._kwargs} 文本')
        element = self.get_element()
        text = element.text
        logger.info(text)
        return text

    def get_texts(self):
        logger.info(f'获取元素 {self._kwargs} 文本列表')
        elements = self.get_elements()
        texts = [elem.text for elem in elements]
        logger.info(texts)
        return texts

    def select_index(self, index):
        logger.info(f'选择第 {index} 个下拉列表')
        element = self.get_element()
        select = Select(element)
        select.select_by_index(index)

    def select_value(self, value):
        logger.info(f'选择id为 {value} 的下拉列表')
        element = self.get_element()
        select = Select(element)
        select.select_by_value(value)

    def select_text(self, text):
        logger.info(f'选择下拉列表 {text} 选项')
        element = self.get_element()
        select = Select(element)
        select.select_by_value(text)

    def submit(self):
        logger.info(f'提交表单: {self._kwargs}')
        elem = self.get_element()
        elem.submit()

    # def screenshot(self, file_name):
    #     """
    #     截图并保存到预定路径并上传到allure报告
    #     @param file_name: foo.png or fool
    #     @return:
    #     """
    #     # 把文件名处理成test.png的样式
    #     if '.' in file_name:
    #         file_name = file_name.split(r'.')[0]
    #     # 截图并保存到当前目录的images文件夹中
    #     img_dir = os.path.join(os.getcwd(), 'images')
    #     if os.path.exists(img_dir) is False:
    #         os.mkdir(img_dir)
    #     time_str = time.strftime('%Y%m%d%H%M%S')
    #     file_path = os.path.join(img_dir,
    #                              f'{file_name}-{time_str}.png')
    #     logger.info(f'截图保存至: {file_path}')
    #     self.get_element().screenshot(file_path)
    #     # 截图并上传到allure报告
    #     logger.info(f'截图上传至allure报告')
    #     allure.attach.file(file_path,
    #                        attachment_type=allure.attachment_type.PNG,
    #                        name=f'{file_name}-{time_str}')





from .. import Provider as AddressProvider


class Provider(AddressProvider):

    # country names adapted from
    # https://github.com/PyThaiNLP/pythainlp/blob/dev/pythainlp/corpus/countries_th.txt
    countries = (
        "กรีซ",
        "กรีนแลนด์",
        "กวม",
        "กัมพูชา",
        "กัวเดอลุป",
        "กัวเตมาลา",
        "กาตาร์",
        "กานา",
        "กาบอง",
        "กาบูเวร์ดี",
        "กายอานา",
        "กินี-บิสเซา",
        "กินี",
        "เกรเนดา",
        "เกาหลีใต้",
        "เกาหลีเหนือ",
        "เกาะคริสต์มาส",
        "เกาะนอร์ฟอล์ก",
        "เกาะบูเวต",
        "เกาะแมน",
        "เกิร์นซีย์",
        "แกมเบีย",
        "โกตดิวัวร์",
        "คอโมโรส",
        "คอสตาริกา",
        "คาซัคสถาน",
        "คิริบาส",
        "คิวบา",
        "คีร์กีซสถาน",
        "คีลิง",
        "คูเวต",
        "เคนยา",
        "แคนาดา",
        "แคเมอรูน",
        "โครเอเชีย",
        "โคลอมเบีย",
        "จอร์เจีย",
        "จอร์แดน",
        "จาเมกา",
        "จิบูตี",
        "จีน",
        "เจอร์ซีย์",
        "ชาด",
        "ชิลี",
        "ซานมารีโน",
        "ซามัว",
        "ซาอุดีอาระเบีย",
        "ซิมบับเว",
        "ซีเรีย",
        "ซูดาน",
        "ซูรินาม",
        "เซเชลส์",
        "เซนต์คิตส์และเนวิส",
        "เซนต์ลูเซีย",
        "เซนต์วินเซนต์และเกรนาดีนส์",
        "เซนต์เฮเลนา",
        "เซเนกัล",
        "เซอร์เบีย",
        "เซาตูเมและปรินซิปี",
        "เซียร์ราลีโอน",
        "แซ็ง-บาร์เตเลมี",
        "แซ็งบาร์เตเลมี",
        "แซ็ง-มาร์แต็ง",
        "แซ็งมาร์แต็ง",
        "แซงปีแยร์และมีเกอลง",
        "แซมเบีย",
        "โซมาเลีย",
        "ไซปรัส",
        "ญี่ปุ่น",
        "ดอมินีกา",
        "เดนมาร์ก",
        "ตรินิแดดและโตเบโก",
        "ตองกา",
        "ติมอร์-เลสเต",
        "ติมอร์เลสเต",
        "ตุรกี",
        "ตูนิเซีย",
        "ตูวาลู",
        "เติร์กเมนิสถาน",
        "โตเกเลา",
        "โตโก",
        "ไต้หวัน",
        "ทาจิกิสถาน",
        "แทนซาเนีย",
        "ไทย",
        "นครรัฐวาติกัน",
        "นอร์เวย์",
        "นามิเบีย",
        "นาอูรู",
        "นิการากัว",
        "นิวแคลิโดเนีย",
        "นิวซีแลนด์",
        "นีอูเอ",
        "เนเธอร์แลนด์แอนทิลลีส",
        "เนเธอร์แลนด์",
        "เนปาล",
        "ไนจีเรีย",
        "ไนเจอร์",
        "บราซิล",
        "บริติชอินเดียนโอเชียนเทร์ริทอรี",
        "บรูไนดารุสซาลาม",
        "บอตสวานา",
        "บอสเนียและเฮอร์เซโกวีนา",
        "บังกลาเทศ",
        "บัลแกเรีย",
        "บาร์เบโดส",
        "บาห์เรน",
        "บาฮามาส",
        "บุรุนดี",
        "บูร์กินาฟาโซ",
        "เบนิน",
        "เบลเยียม",
        "เบลารุส",
        "เบลีซ",
        "เบอร์มิวดา",
        "โบลิเวีย",
        "ปากีสถาน",
        "ปานามา",
        "ปาปัวนิวกินี",
        "ปารากวัย",
        "ปาเลา",
        "ปาเลสไตน์",
        "เปรู",
        "เปอร์โตริโก",
        "โปรตุเกส",
        "โปแลนด์",
        "ฝรั่งเศส",
        "พม่า",
        "ฟิจิ",
        "ฟินแลนด์",
        "ฟิลิปปินส์",
        "เฟรนช์เกียนา",
        "เฟรนช์โปลินีเซีย",
        "ภูฏาน",
        "มองโกเลีย",
        "มอนต์เซอร์รัต",
        "มอนเตเนโกร",
        "มอริเชียส",
        "มอริเตเนีย",
        "มอลโดวา",
        "มอลตา",
        "มัลดีฟส์",
        "มาเก๊า",
        "มาซิโดเนีย",
        "มาดากัสการ์",
        "มายอต",
        "มาร์ตีนิก",
        "มาลาวี",
        "มาลี",
        "มาเลเซีย",
        "เม็กซิโก",
        "โมซัมบิก",
        "โมนาโก",
        "โมร็อกโก",
        "ไมโครนีเซีย",
        "ยานไมเอน",
        "ยิบรอลตาร์",
        "ยูกันดา",
        "ยูเครน",
        "เยเมน",
        "เยอรมนี",
        "รวันดา",
        "รัสเซีย",
        "เรอูว์นียง",
        "โรมาเนีย",
        "ลักเซมเบิร์ก",
        "ลัตเวีย",
        "ลาว",
        "ลิกเตนสไตน์",
        "ลิทัวเนีย",
        "ลิเบีย",
        "เลโซโท",
        "เลบานอน",
        "ไลบีเรีย",
        "วานูอาตู",
        "วาลลิสและฟุตูนา",
        "เวเนซุเอลา",
        "เวสเทิร์นสะฮารา",
        "เวียดนาม",
        "ศรีลังกา",
        "สกอตแลนด์",
        "สเปน",
        "สฟาลบาร์",
        "สโลวาเกีย",
        "สโลวีเนีย",
        "สวาซิแลนด์",
        "สวิตเซอร์แลนด์",
        "สวีเดน",
        "สหรัฐอเมริกา",
        "สหรัฐอาหรับเอมิเรตส์",
        "สหราชอาณาจักร",
        "สาธารณรัฐคองโก",
        "สาธารณรัฐเช็ก",
        "สาธารณรัฐโดมินิกัน",
        "สิงคโปร์",
        "หมู่เกาะคุก",
        "หมู่เกาะเคย์แมน",
        "หมู่เกาะโคโคส",
        "หมู่เกาะโซโลมอน",
        "หมู่เกาะบริติชเวอร์จิน",
        "หมู่เกาะพิตแคร์น",
        "หมู่เกาะฟอล์กแลนด์",
        "หมู่เกาะแฟโร",
        "หมู่เกาะมาร์แชลล์",
        "อเมริกันซามัว",
        "ออสเตรเลีย",
        "ออสเตรีย",
        "อังกฤษ",
        "อันดอร์รา",
        "อัฟกานิสถาน",
        "อาเซอร์ไบจาน",
        "อาร์เจนตินา",
        "อาร์มีเนีย",
        "อารูบา",
        "อิเควทอเรียลกินี",
        "อิตาลี",
        "อินเดีย",
        "อินโดนีเซีย",
        "อิรัก",
        "อิสราเอล",
        "อิหร่าน",
        "อียิปต์",
        "อุซเบกิสถาน",
        "อุรุกวัย",
        "เอกวาดอร์",
        "เอธิโอเปีย",
        "เอริเทรีย",
        "เอลซัลวาดอร์",
        "เอสโตเนีย",
        "แองกวิลลา",
        "แองโกลา",
        "แอฟริกากลาง",
        "แอฟริกาใต้",
        "แอลจีเรีย",
        "แอลเบเนีย",
        "โอมาน",
        "ไอซ์แลนด์",
        "ไอร์แลนด์",
        "ไอวอรีโคสต์",
        "ฮ่องกง",
        "ฮอนดูรัส",
        "ฮังการี",
        "เฮติ",
    )
